local StoneGateBridge = ...

--------------------------------------------------------------------------
local STONE_GATE_BRIDGE_COMP = {
	TypeName = DrawBridgePrefix .. "_COMP",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "DrawBridge", Type = "string", Default = "A" },
		{ Name = "MinDistanceVillager", Type = "integer", Default = 50 },
		{ Name = "MaxDistanceVillager", Type = "integer", Default = 100 },
		{ Name = "MinDistanceShip", Type = "integer", Default = 100 },
		{ Name = "MaxDistanceShip", Type = "integer", Default = 150 },
                { Name = "HasBoat", Type = "boolean", Default = false}
	}
}

--------------------------------------------------------------------------
function STONE_GATE_BRIDGE_COMP:create()
	self.timer = 0
        self._rotationX = math.pi/2
        self.isOpened = true
end

--------------------------------------------------------------------------
function STONE_GATE_BRIDGE_COMP:computePositionLocator(_locatorName)
        local pos1 = nil
        self:getOwner():forEachChild(
            function(child)
                 if child.Name == _locatorName then
                     pos1 = child:getGlobalPosition()
                 end
            end
        )
        if pos1 == nil then
            StoneGateBridge:logError(_locatorName .. " missing")
        end
        return pos1
end
function STONE_GATE_BRIDGE_COMP:init()
        self.pausedAgent = {}
        self.positionAgent = {}
        self.pos1 = self:computePositionLocator("Pont_Center_Locator")
end

--------------------------------------------------------------------------
function STONE_GATE_BRIDGE_COMP:raiseChaineA(_angle)
    if _angle <=  math.pi/6 then
        self:getOwner():forEachChildRecursive(
            function(child)
                if child.Name == "pont_sgv5dba" then
                    child:setRotationX(_angle)
                elseif (child.Name == "chaine1_sgv5dba") or (child.Name == "chaine2_sgv5dba") then
                    child:setRotationX(-_angle/2)
                elseif (child.Name == "maillons2_sgv5dba") or (child.Name == "maillons3_sgv5dba") then
                    child:setRotationX(0)
                end
            end     
        )
    elseif _angle <=  0.27*math.pi then
        self:getOwner():forEachChildRecursive(
            function(child)
                if child.Name == "pont_sgv5dba" then
                    child:setRotationX(_angle)
                elseif (child.Name == "chaine1_sgv5dba") or (child.Name == "chaine2_sgv5dba") then
                    child:setRotationX(-_angle/2)
                elseif child.Name == "maillons2_sgv5dba" then
                    child:setRotationX(0)
                elseif child.Name == "maillons3_sgv5dba" then
                    child:setRotationX(3*math.pi/4 -_angle/2)
                end
            end     
        )
    else
        self:getOwner():forEachChildRecursive(
            function(child)
                if child.Name == "pont_sgv5dba" then
                    child:setRotationX(_angle)
                elseif (child.Name == "chaine1_sgv5dba") or (child.Name == "chaine2_sgv5dba") then
                    child:setRotationX(-_angle/2)
                elseif child.Name == "maillons2_sgv5dba" then
                    child:setRotationX(3*math.pi/4 -_angle/2)
                elseif child.Name == "maillons3_sgv5dba" then
                    child:setRotationX(0)
                end
            end     
        )
    end
end

--------------------------------------------------------------------------
function STONE_GATE_BRIDGE_COMP:raiseChaineB(_angle)
    self:getOwner():forEachChildRecursive(
        function(child)
            if (child.Name == "pont_sgv5dbb") or (child.Name == "fleches_sgv5dbb") then
                child:setRotationX(_angle)
            elseif (child.Name == "chaine1_sgv5dbb") or (child.Name == "chaine2_sgv5dbb") then
                child:setRotationX(-_angle)
            end
        end     
    )
end

--------------------------------------------------------------------------
function STONE_GATE_BRIDGE_COMP:raiseChaineC(_angle)
    self:getOwner():forEachChildRecursive(
        function(child)
            if (child.Name == "pont_sgv5dbc") or (child.Name == "fleches_sgv5dbc") then
                child:setRotationX(_angle)
            elseif (child.Name == "chaine1_sgv5dbc") or (child.Name == "chaine2_sgv5dbc") then
                child:setRotationX(-_angle)
            elseif (child.Name == "pont1_sgv5dbc") or (child.Name == "fleches1_sgv5dbc") then
                child:setRotationX(-_angle)
            elseif (child.Name == "chaine3_sgv5dbc") or (child.Name == "chaine4_sgv5dbc") then
                child:setRotationX(_angle)
            end
        end     
    )
end

--------------------------------------------------------------------------
function STONE_GATE_BRIDGE_COMP:bridgeClose(_dt)
    self._rotationX = self._rotationX + _dt*math.pi/3
    if self._rotationX > 7*math.pi/16 then
        self._rotationX = 7*math.pi/16
    end
    if self.DrawBridge == "A" then
        self:raiseChaineA(self._rotationX)
    elseif self.DrawBridge == "B" then
        self:raiseChaineB(self._rotationX)
    elseif self.DrawBridge == "C" then
        self:raiseChaineC(self._rotationX)
    end        
end

--------------------------------------------------------------------------
function STONE_GATE_BRIDGE_COMP:bridgeOpen(_dt)
    self._rotationX = self._rotationX - _dt*math.pi/2
    if self._rotationX < 0 then
        self._rotationX = 0
    end
    if self.DrawBridge == "A" then
        self:raiseChaineA(self._rotationX)        
    elseif self.DrawBridge == "B" then
        self:raiseChaineB(self._rotationX)        
    elseif self.DrawBridge == "C" then
        self:raiseChaineC(self._rotationX)        
    end
    return self._rotationX <= 0
end

--------------------------------------------------------------------------
function STONE_GATE_BRIDGE_COMP:computeProximity(_comp, _pos1)
                    local pos2 = _comp:getOwner():getGlobalPosition()
                    local distance = (_pos1.x - pos2.x)^2 + (_pos1.z - pos2.z)^2
                    return distance
end
function STONE_GATE_BRIDGE_COMP:update()
    if self.pos1 == nil then
        return
    end
    local dt = self:getLevel():getDeltaTime()
    self.timer = self.timer - dt
    if self.timer < 0 then -- in case of pause (dt==0) nothing is executed
        -- check the proximity of a boat
        local _proximity = false
        if (self.HasBoat) then
            local componentHarbour = self:getLevel():getComponentManager("HARBOUR_V2_COMP_BOAT")
            if (componentHarbour ~= nil) then
                componentHarbour:getAllComponent():forEach(
                    function(comp)
                        if self:computeProximity(comp, self.pos1) < self.MinDistanceShip then
                            _proximity = true
                            self:bridgeClose(dt)
                            self.timer = 0 -- to recheck the proximity on each tick, wether execute the smooth closing rotation
                        elseif self:computeProximity(comp, self.pos1) < self.MaxDistanceShip then -- to let the villager already on the bridge finish their crossing
                            _proximity = true
                            self.timer = 0 -- to recheck the proximity on each tick, wether execute the smooth closing rotation
                        end
                    end
                )
            end
        end
        if _proximity then
--            StoneGateBridge:log("STONE_GATE_BRIDGE_COMP:update() pause agents")
            self:getLevel():getComponentManager("COMP_AGENT"):getAllComponent():forEach(
                function(comp)
--                    StoneGateBridge:log("STONE_GATE_BRIDGE_COMP:update() check job agent " .. comp:getOwner().Name)
                    local _villager = comp:getOwner():getEnabledComponent("COMP_VILLAGER")

                    if (_villager ~= nil) then
                        if (_villager:getAssetJob() ~= nil) and (_villager:getAssetJob().JobName == "HARBOUR_V2_SAILOR_NAME") then
 --                           StoneGateBridge:log("STONE_GATE_BRIDGE_COMP:update() sailor detected " .. comp:getOwner().Name .. " " .. tostring(_villager:getAssetJob().JobName))
                            return;
                        end
--                        StoneGateBridge:log("STONE_GATE_BRIDGE_COMP:update() check proximity " .. comp:getOwner().Name)
                        if (self:computeProximity(comp, self.pos1) < self.MinDistanceVillager) then
                            for i,agent in ipairs(self.pausedAgent) do
                                if agent == comp:getOwner().Name then
--                                    StoneGateBridge:log("STONE_GATE_BRIDGE_COMP:update() restore position agent " .. comp:getOwner().Name)
                                    if self.positionAgent[i] == nil then -- approach detected
                                        self.positionAgent[i] = comp:getOwner():getGlobalPosition()
--                                        StoneGateBridge:log("STONE_GATE_BRIDGE_COMP:update() memorization 2 position agent " .. comp:getOwner().Name .. " " .. tostring(_villager:getAssetJob().JobName))
                                    else --villager waiting 
                                        comp:getOwner():setGlobalPosition(self.positionAgent[i])
                                    end
                                end
                            end
                        
                        elseif (self:computeProximity(comp, self.pos1) < self.MaxDistanceVillager) then
                            local trouve = false
                            for i,agent in ipairs(self.pausedAgent) do
                                if agent == comp:getOwner().Name then -- let him continue
                                    trouve = true
                                end
                            end
                            if not trouve then
--                                StoneGateBridge:log("STONE_GATE_BRIDGE_COMP:update() proximity agent detected " .. comp:getOwner().Name .. " " .. tostring(_villager:getAssetJob().JobName))
                                table.insert(self.pausedAgent, comp:getOwner().Name)
                                table.insert(self.positionAgent, nil) -- proximity detected
                            end
                        end
                    end
                end
            )
        else
--            if #self.pausedAgent > 0 then 
--                StoneGateBridge:log("STONE_GATE_BRIDGE_COMP:update() clear pausedAgent ")
--            end
            self.pausedAgent = {}
            self.positionAgent = {}
            -- check the proximity of a villager 
            local _proximity = false
  
            self:getLevel():getComponentManager("COMP_AGENT"):getAllComponent():forEach(
                function(comp)
                    -- if proximity detected, bridgeOpen is executed only once
                    if not _proximity then
                        if self:computeProximity(comp, self.pos1) < self.MinDistanceVillager then
                            _proximity = true
                            self:bridgeOpen(dt)
                            self.timer = 0 -- to execute the smooth opening rotation on each tick
                        end
                    end
                end
            )
            if not _proximity then
                self:bridgeClose(dt)
                self.timer = 0 -- to recheck the proximity on each tick, wether execute the smooth closing rotation
            end
        end 
    end
end

--------------------------------------------------------------------------
StoneGateBridge:registerClass(STONE_GATE_BRIDGE_COMP)
